@@ -86,14 +86,30 @@ Creates a new double per sample image.
 =cut
 */
 i_img *i_img_double_new_low(i_img *im, int x, int y, int ch) {
+  int bytes;
+
   mm_log((1,"i_img_double_new(x %d, y %d, ch %d)\n", x, y, ch));
+
+  if (x < 1 || y < 1) {
+    i_push_error(0, "Image sizes must be positive");
+    return NULL;
+  }
+  if (ch < 1 || ch > MAXCHANNELS) {
+    i_push_errorf(0, "channels must be between 1 and %d", MAXCHANNELS);
+    return NULL;
+  }
+  bytes = x * y * ch * sizeof(double);
+  if (bytes / y / ch / sizeof(double) != x) {
+    i_push_errorf(0, "integer overflow calculating image allocation");
+    return NULL;
+  }
   
   *im = IIM_base_double_direct;
   i_tags_new(&im->tags);
   im->xsize = x;
   im->ysize = y;
   im->channels = ch;
-  im->bytes = x * y * ch * sizeof(double);
+  im->bytes = bytes;
   im->ext_data = NULL;
   im->idata = mymalloc(im->bytes);
   if (im->idata) {
@@ -110,6 +126,8 @@ i_img *i_img_double_new_low(i_img *im, int x, int y, int ch) {
 i_img *i_img_double_new(int x, int y, int ch) {
   i_img *im;
 
+  i_clear_error();
+
   im = mymalloc(sizeof(i_img));
   if (im) {
     if (!i_img_double_new_low(im, x, y, ch)) {
@@ -130,8 +148,15 @@ static int i_ppix_ddoub(i_img *im, int x, int y, i_color *val) {
     return -1;
 
   off = (x + y * im->xsize) * im->channels;
-  for (ch = 0; ch < im->channels; ++ch)
-    ((double*)im->idata)[off+ch] = Sample8ToF(val->channel[ch]);
+  if (I_ALL_CHANNELS_WRITABLE(im)) {
+    for (ch = 0; ch < im->channels; ++ch)
+      ((double*)im->idata)[off+ch] = Sample8ToF(val->channel[ch]);
+  }
+  else {
+    for (ch = 0; ch < im->channels; ++ch)
+      if (im->ch_mask & (1<<ch))
+	((double*)im->idata)[off+ch] = Sample8ToF(val->channel[ch]);
+  }
 
   return 0;
 }
@@ -156,8 +181,15 @@ static int i_ppixf_ddoub(i_img *im, int x, int y, i_fcolor *val) {
     return -1;
 
   off = (x + y * im->xsize) * im->channels;
-  for (ch = 0; ch < im->channels; ++ch)
-    ((double *)im->idata)[off+ch] = val->channel[ch];;
+  if (I_ALL_CHANNELS_WRITABLE(im)) {
+    for (ch = 0; ch < im->channels; ++ch)
+      ((double *)im->idata)[off+ch] = val->channel[ch];
+  }
+  else {
+    for (ch = 0; ch < im->channels; ++ch)
+      if (im->ch_mask & (1 << ch))
+	((double *)im->idata)[off+ch] = val->channel[ch];
+  }
 
   return 0;
 }
@@ -204,10 +236,21 @@ static int i_plin_ddoub(i_img *im, int l, int r, int y, i_color *vals) {
       r = im->xsize;
     off = (l+y*im->xsize) * im->channels;
     count = r - l;
-    for (i = 0; i < count; ++i) {
-      for (ch = 0; ch < im->channels; ++ch) {
-        ((double *)im->idata)[off] = Sample8ToF(vals[i].channel[ch]);
-        ++off;
+    if (I_ALL_CHANNELS_WRITABLE(im)) {
+      for (i = 0; i < count; ++i) {
+	for (ch = 0; ch < im->channels; ++ch) {
+	  ((double *)im->idata)[off] = Sample8ToF(vals[i].channel[ch]);
+	  ++off;
+	}
+      }
+    }
+    else {
+      for (i = 0; i < count; ++i) {
+	for (ch = 0; ch < im->channels; ++ch) {
+	  if (im->ch_mask & (1 << ch))
+	    ((double *)im->idata)[off] = Sample8ToF(vals[i].channel[ch]);
+	  ++off;
+	}
       }
     }
     return count;
@@ -246,10 +289,21 @@ static int i_plinf_ddoub(i_img *im, int l, int r, int y, i_fcolor *vals) {
       r = im->xsize;
     off = (l+y*im->xsize) * im->channels;
     count = r - l;
-    for (i = 0; i < count; ++i) {
-      for (ch = 0; ch < im->channels; ++ch) {
-        ((double *)im->idata)[off] = vals[i].channel[ch];
-        ++off;
+    if (I_ALL_CHANNELS_WRITABLE(im)) {
+      for (i = 0; i < count; ++i) {
+	for (ch = 0; ch < im->channels; ++ch) {
+	  ((double *)im->idata)[off] = vals[i].channel[ch];
+	  ++off;
+	}
+      }
+    }
+    else {
+      for (i = 0; i < count; ++i) {
+	for (ch = 0; ch < im->channels; ++ch) {
+	  if (im->ch_mask & (1 << ch))
+	    ((double *)im->idata)[off] = vals[i].channel[ch];
+	  ++off;
+	}
       }
     }
     return count;
