@@ -1,4 +1,4 @@
-#include "io.h"
+#include "imio.h"
 #include <stdlib.h>
 #ifndef _MSC_VER
 #include <unistd.h>
@@ -115,12 +115,6 @@ mymalloc_file_line(size_t size, char* file, int line) {
   return buf;
 }
 
-
-
-
-
-
-
 void*
 myrealloc_file_line(void *ptr, size_t newsize, char* file, int line) {
   char *buf;
@@ -155,9 +149,6 @@ myrealloc_file_line(void *ptr, size_t newsize, char* file, int line) {
   return buf;
 }
 
-
-
-
 static
 void
 bndcheck(int idx) {
@@ -229,6 +220,11 @@ void*
 mymalloc(int size) {
   void *buf;
 
+  if (size < 0) {
+    fprintf(stderr, "Attempt to allocate size %d\n", size);
+    exit(3);
+  }
+
   if ( (buf = malloc(size)) == NULL ) {
     mm_log((1, "mymalloc: unable to malloc %d\n", size));
     fprintf(stderr,"Unable to malloc %d.\n", size); exit(3);
